@@ -7,6 +7,7 @@ package runtime
 
 import (
 	"reflect"
+	"tinygo"
 	"unsafe"
 )
 
@@ -15,21 +16,13 @@ type hashmap struct {
 	buckets    unsafe.Pointer // pointer to array of buckets
 	seed       uintptr
 	count      uintptr
-	keySize    uint8 // maybe this can store the key type as well? E.g. keysize == 5 means string?
-	valueSize  uint8
+	keySize    uintptr // maybe this can store the key type as well? E.g. keysize == 5 means string?
+	valueSize  uintptr
 	bucketBits uint8
 	keyEqual   func(x, y unsafe.Pointer, n uintptr) bool
 	keyHash    func(key unsafe.Pointer, size, seed uintptr) uint32
 }
 
-type hashmapAlgorithm uint8
-
-const (
-	hashmapAlgorithmBinary hashmapAlgorithm = iota
-	hashmapAlgorithmString
-	hashmapAlgorithmInterface
-)
-
 // A hashmap bucket. A bucket is a container of 8 key/value pairs: first the
 // following two entries, then the 8 keys, then the 8 values. This somewhat odd
 // ordering is to make sure the keys and values are well aligned when one of
@@ -45,8 +38,15 @@ type hashmapIterator struct {
 	buckets      unsafe.Pointer // pointer to array of hashapBuckets
 	numBuckets   uintptr        // length of buckets array
 	bucketNumber uintptr        // current index into buckets array
+	startBucket  uintptr        // starting location for iterator
 	bucket       *hashmapBucket // current bucket in chain
 	bucketIndex  uint8          // current index into bucket
+	startIndex   uint8          // starting bucket index for iterator
+	wrapped      bool           // true if the iterator has wrapped
+}
+
+func hashmapNewIterator() unsafe.Pointer {
+	return unsafe.Pointer(new(hashmapIterator))
 }
 
 // Get the topmost 8 bits of the hash, without using a special value (like 0).
@@ -60,18 +60,17 @@ func hashmapTopHash(hash uint32) uint8 {
 }
 
 // Create a new hashmap with the given keySize and valueSize.
-func hashmapMake(keySize, valueSize uint8, sizeHint uintptr, alg uint8) *hashmap {
-	numBuckets := sizeHint / 8
+func hashmapMake(keySize, valueSize uintptr, sizeHint uintptr, alg uint8) *hashmap {
 	bucketBits := uint8(0)
-	for numBuckets != 0 {
-		numBuckets /= 2
+	for hashmapHasSpaceToGrow(bucketBits) && hashmapOverLoadFactor(sizeHint, bucketBits) {
 		bucketBits++
 	}
-	bucketBufSize := unsafe.Sizeof(hashmapBucket{}) + uintptr(keySize)*8 + uintptr(valueSize)*8
+
+	bucketBufSize := unsafe.Sizeof(hashmapBucket{}) + keySize*8 + valueSize*8
 	buckets := alloc(bucketBufSize*(1<<bucketBits), nil)
 
-	keyHash := hashmapKeyHashAlg(hashmapAlgorithm(alg))
-	keyEqual := hashmapKeyEqualAlg(hashmapAlgorithm(alg))
+	keyHash := hashmapKeyHashAlg(tinygo.HashmapAlgorithm(alg))
+	keyEqual := hashmapKeyEqualAlg(tinygo.HashmapAlgorithm(alg))
 
 	return &hashmap{
 		buckets:    buckets,
@@ -84,13 +83,42 @@ func hashmapMake(keySize, valueSize uint8, sizeHint uintptr, alg uint8) *hashmap
 	}
 }
 
-func hashmapKeyEqualAlg(alg hashmapAlgorithm) func(x, y unsafe.Pointer, n uintptr) bool {
+// Remove all entries from the map, without actually deallocating the space for
+// it. This is used for the clear builtin, and can be used to reuse a map (to
+// avoid extra heap allocations).
+func hashmapClear(m *hashmap) {
+	if m == nil {
+		// Nothing to do. According to the spec:
+		// > If the map or slice is nil, clear is a no-op.
+		return
+	}
+
+	m.count = 0
+	numBuckets := uintptr(1) << m.bucketBits
+	bucketSize := hashmapBucketSize(m)
+	for i := uintptr(0); i < numBuckets; i++ {
+		bucket := hashmapBucketAddr(m, m.buckets, i)
+		for bucket != nil {
+			// Clear the tophash, to mark these keys/values as removed.
+			bucket.tophash = [8]uint8{}
+
+			// Clear the keys and values in the bucket so that the GC won't pin
+			// these allocations.
+			memzero(unsafe.Add(unsafe.Pointer(bucket), unsafe.Sizeof(hashmapBucket{})), bucketSize-unsafe.Sizeof(hashmapBucket{}))
+
+			// Move on to the next bucket in the chain.
+			bucket = bucket.next
+		}
+	}
+}
+
+func hashmapKeyEqualAlg(alg tinygo.HashmapAlgorithm) func(x, y unsafe.Pointer, n uintptr) bool {
 	switch alg {
-	case hashmapAlgorithmBinary:
+	case tinygo.HashmapAlgorithmBinary:
 		return memequal
-	case hashmapAlgorithmString:
+	case tinygo.HashmapAlgorithmString:
 		return hashmapStringEqual
-	case hashmapAlgorithmInterface:
+	case tinygo.HashmapAlgorithmInterface:
 		return hashmapInterfaceEqual
 	default:
 		// compiler bug :(
@@ -98,13 +126,13 @@ func hashmapKeyEqualAlg(alg hashmapAlgorithm) func(x, y unsafe.Pointer, n uintpt
 	}
 }
 
-func hashmapKeyHashAlg(alg hashmapAlgorithm) func(key unsafe.Pointer, n, seed uintptr) uint32 {
+func hashmapKeyHashAlg(alg tinygo.HashmapAlgorithm) func(key unsafe.Pointer, n, seed uintptr) uint32 {
 	switch alg {
-	case hashmapAlgorithmBinary:
+	case tinygo.HashmapAlgorithmBinary:
 		return hash32
-	case hashmapAlgorithmString:
+	case tinygo.HashmapAlgorithmString:
 		return hashmapStringPtrHash
-	case hashmapAlgorithmInterface:
+	case tinygo.HashmapAlgorithmInterface:
 		return hashmapInterfacePtrHash
 	default:
 		// compiler bug :(
@@ -112,14 +140,14 @@ func hashmapKeyHashAlg(alg hashmapAlgorithm) func(key unsafe.Pointer, n, seed ui
 	}
 }
 
-func hashmapShouldGrow(m *hashmap) bool {
-	if m.bucketBits > uint8((unsafe.Sizeof(uintptr(0))*8)-3) {
-		// Over this limit, we're likely to overflow uintptrs during calculations
-		// or numbers of hash elements.   Don't allow any more growth.
-		// With 29 bits, this is 2^32 elements anyway.
-		return false
-	}
+func hashmapHasSpaceToGrow(bucketBits uint8) bool {
+	// Over this limit, we're likely to overflow uintptrs during calculations
+	// or numbers of hash elements.   Don't allow any more growth.
+	// With 29 bits, this is 2^32 elements anyway.
+	return bucketBits <= uint8((unsafe.Sizeof(uintptr(0))*8)-3)
+}
 
+func hashmapOverLoadFactor(n uintptr, bucketBits uint8) bool {
 	// "maximum" number of elements is 0.75 * buckets * elements per bucket
 	// to avoid overflow, this is calculated as
 	// max = 3 * (1/4 * buckets * elements per bucket)
@@ -127,12 +155,13 @@ func hashmapShouldGrow(m *hashmap) bool {
 	//     = 3 * (buckets * (8/4)
 	//     = 3 * (buckets * 2)
 	//     = 6 * buckets
-	max := (uintptr(6) << m.bucketBits)
-	return m.count > max
+	max := (uintptr(6) << bucketBits)
+	return n > max
 }
 
 // Return the number of entries in this hashmap, called from the len builtin.
 // A nil hashmap is defined as having length 0.
+//
 //go:inline
 func hashmapLen(m *hashmap) int {
 	if m == nil {
@@ -141,28 +170,51 @@ func hashmapLen(m *hashmap) int {
 	return int(m.count)
 }
 
-// wrapper for use in reflect
-func hashmapLenUnsafePointer(p unsafe.Pointer) int {
-	m := (*hashmap)(p)
-	return hashmapLen(m)
+//go:inline
+func hashmapBucketSize(m *hashmap) uintptr {
+	return unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*8
+}
+
+//go:inline
+func hashmapBucketAddr(m *hashmap, buckets unsafe.Pointer, n uintptr) *hashmapBucket {
+	bucketSize := hashmapBucketSize(m)
+	bucket := (*hashmapBucket)(unsafe.Add(buckets, bucketSize*n))
+	return bucket
+}
+
+//go:inline
+func hashmapBucketAddrForHash(m *hashmap, hash uint32) *hashmapBucket {
+	numBuckets := uintptr(1) << m.bucketBits
+	bucketNumber := (uintptr(hash) & (numBuckets - 1))
+	return hashmapBucketAddr(m, m.buckets, bucketNumber)
+}
+
+//go:inline
+func hashmapSlotKey(m *hashmap, bucket *hashmapBucket, slot uint8) unsafe.Pointer {
+	slotKeyOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*uintptr(slot)
+	slotKey := unsafe.Add(unsafe.Pointer(bucket), slotKeyOffset)
+	return slotKey
+}
+
+//go:inline
+func hashmapSlotValue(m *hashmap, bucket *hashmapBucket, slot uint8) unsafe.Pointer {
+	slotValueOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*uintptr(slot)
+	slotValue := unsafe.Add(unsafe.Pointer(bucket), slotValueOffset)
+	return slotValue
 }
 
 // Set a specified key to a given value. Grow the map if necessary.
+//
 //go:nobounds
 func hashmapSet(m *hashmap, key unsafe.Pointer, value unsafe.Pointer, hash uint32) {
-	if hashmapShouldGrow(m) {
+	if hashmapHasSpaceToGrow(m.bucketBits) && hashmapOverLoadFactor(m.count, m.bucketBits) {
 		hashmapGrow(m)
 		// seed changed when we grew; rehash key with new seed
-		hash = m.keyHash(key, uintptr(m.keySize), m.seed)
+		hash = m.keyHash(key, m.keySize, m.seed)
 	}
 
 	tophash := hashmapTopHash(hash)
-
-	numBuckets := uintptr(1) << m.bucketBits
-	bucketNumber := (uintptr(hash) & (numBuckets - 1))
-	bucketSize := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*8
-	bucketAddr := uintptr(m.buckets) + bucketSize*bucketNumber
-	bucket := (*hashmapBucket)(unsafe.Pointer(bucketAddr))
+	bucket := hashmapBucketAddrForHash(m, hash)
 	var lastBucket *hashmapBucket
 
 	// See whether the key already exists somewhere.
@@ -170,11 +222,9 @@ func hashmapSet(m *hashmap, key unsafe.Pointer, value unsafe.Pointer, hash uint3
 	var emptySlotValue unsafe.Pointer
 	var emptySlotTophash *byte
 	for bucket != nil {
-		for i := uintptr(0); i < 8; i++ {
-			slotKeyOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*uintptr(i)
-			slotKey := unsafe.Pointer(uintptr(unsafe.Pointer(bucket)) + slotKeyOffset)
-			slotValueOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*uintptr(i)
-			slotValue := unsafe.Pointer(uintptr(unsafe.Pointer(bucket)) + slotValueOffset)
+		for i := uint8(0); i < 8; i++ {
+			slotKey := hashmapSlotKey(m, bucket, i)
+			slotValue := hashmapSlotValue(m, bucket, i)
 			if bucket.tophash[i] == 0 && emptySlotKey == nil {
 				// Found an empty slot, store it for if we couldn't find an
 				// existing slot.
@@ -184,9 +234,9 @@ func hashmapSet(m *hashmap, key unsafe.Pointer, value unsafe.Pointer, hash uint3
 			}
 			if bucket.tophash[i] == tophash {
 				// Could be an existing key that's the same.
-				if m.keyEqual(key, slotKey, uintptr(m.keySize)) {
+				if m.keyEqual(key, slotKey, m.keySize) {
 					// found same key, replace it
-					memcpy(slotValue, value, uintptr(m.valueSize))
+					memcpy(slotValue, value, m.valueSize)
 					return
 				}
 			}
@@ -201,56 +251,69 @@ func hashmapSet(m *hashmap, key unsafe.Pointer, value unsafe.Pointer, hash uint3
 		return
 	}
 	m.count++
-	memcpy(emptySlotKey, key, uintptr(m.keySize))
-	memcpy(emptySlotValue, value, uintptr(m.valueSize))
+	memcpy(emptySlotKey, key, m.keySize)
+	memcpy(emptySlotValue, value, m.valueSize)
 	*emptySlotTophash = tophash
 }
 
 // hashmapInsertIntoNewBucket creates a new bucket, inserts the given key and
 // value into the bucket, and returns a pointer to this bucket.
 func hashmapInsertIntoNewBucket(m *hashmap, key, value unsafe.Pointer, tophash uint8) *hashmapBucket {
-	bucketBufSize := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*8
+	bucketBufSize := hashmapBucketSize(m)
 	bucketBuf := alloc(bucketBufSize, nil)
-	// Insert into the first slot, which is empty as it has just been allocated.
-	slotKeyOffset := unsafe.Sizeof(hashmapBucket{})
-	slotKey := unsafe.Pointer(uintptr(bucketBuf) + slotKeyOffset)
-	slotValueOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8
-	slotValue := unsafe.Pointer(uintptr(bucketBuf) + slotValueOffset)
-	m.count++
-	memcpy(slotKey, key, uintptr(m.keySize))
-	memcpy(slotValue, value, uintptr(m.valueSize))
 	bucket := (*hashmapBucket)(bucketBuf)
+
+	// Insert into the first slot, which is empty as it has just been allocated.
+	slotKey := hashmapSlotKey(m, bucket, 0)
+	slotValue := hashmapSlotValue(m, bucket, 0)
+	m.count++
+	memcpy(slotKey, key, m.keySize)
+	memcpy(slotValue, value, m.valueSize)
 	bucket.tophash[0] = tophash
 	return bucket
 }
 
 func hashmapGrow(m *hashmap) {
+	// allocate our new buckets twice as big
+	n := hashmapCopy(m, m.bucketBits+1)
+	*m = n
+}
+
+//go:linkname hashmapClone maps.clone
+func hashmapClone(intf _interface) _interface {
+	typ, val := decomposeInterface(intf)
+	m := (*hashmap)(val)
+	n := hashmapCopy(m, m.bucketBits)
+	return composeInterface(typ, unsafe.Pointer(&n))
+}
+
+func hashmapCopy(m *hashmap, sizeBits uint8) hashmap {
 	// clone map as empty
 	n := *m
 	n.count = 0
 	n.seed = uintptr(fastrand())
 
-	// allocate our new buckets twice as big
-	n.bucketBits = m.bucketBits + 1
+	n.bucketBits = sizeBits
 	numBuckets := uintptr(1) << n.bucketBits
-	bucketBufSize := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*8
+	bucketBufSize := hashmapBucketSize(m)
 	n.buckets = alloc(bucketBufSize*numBuckets, nil)
 
 	// use a hashmap iterator to go through the old map
 	var it hashmapIterator
 
-	var key = alloc(uintptr(m.keySize), nil)
-	var value = alloc(uintptr(m.valueSize), nil)
+	var key = alloc(m.keySize, nil)
+	var value = alloc(m.valueSize, nil)
 
 	for hashmapNext(m, &it, key, value) {
 		h := n.keyHash(key, uintptr(n.keySize), n.seed)
 		hashmapSet(&n, key, value, h)
 	}
 
-	*m = n
+	return n
 }
 
 // Get the value of a specified key, or zero the value if not found.
+//
 //go:nobounds
 func hashmapGet(m *hashmap, key, value unsafe.Pointer, valueSize uintptr, hash uint32) bool {
 	if m == nil {
@@ -260,30 +323,20 @@ func hashmapGet(m *hashmap, key, value unsafe.Pointer, valueSize uintptr, hash u
 		memzero(value, uintptr(valueSize))
 		return false
 	}
-	numBuckets := uintptr(1) << m.bucketBits
-	bucketNumber := (uintptr(hash) & (numBuckets - 1))
-	bucketSize := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*8
-	bucketAddr := uintptr(m.buckets) + bucketSize*bucketNumber
-	bucket := (*hashmapBucket)(unsafe.Pointer(bucketAddr))
 
-	tophash := uint8(hash >> 24)
-	if tophash < 1 {
-		// 0 means empty slot, so make it bigger.
-		tophash += 1
-	}
+	tophash := hashmapTopHash(hash)
+	bucket := hashmapBucketAddrForHash(m, hash)
 
 	// Try to find the key.
 	for bucket != nil {
-		for i := uintptr(0); i < 8; i++ {
-			slotKeyOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*uintptr(i)
-			slotKey := unsafe.Pointer(uintptr(unsafe.Pointer(bucket)) + slotKeyOffset)
-			slotValueOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*uintptr(i)
-			slotValue := unsafe.Pointer(uintptr(unsafe.Pointer(bucket)) + slotValueOffset)
+		for i := uint8(0); i < 8; i++ {
+			slotKey := hashmapSlotKey(m, bucket, i)
+			slotValue := hashmapSlotValue(m, bucket, i)
 			if bucket.tophash[i] == tophash {
 				// This could be the key we're looking for.
-				if m.keyEqual(key, slotKey, uintptr(m.keySize)) {
+				if m.keyEqual(key, slotKey, m.keySize) {
 					// Found the key, copy it.
-					memcpy(value, slotValue, uintptr(m.valueSize))
+					memcpy(value, slotValue, m.valueSize)
 					return true
 				}
 			}
@@ -292,12 +345,13 @@ func hashmapGet(m *hashmap, key, value unsafe.Pointer, valueSize uintptr, hash u
 	}
 
 	// Did not find the key.
-	memzero(value, uintptr(m.valueSize))
+	memzero(value, m.valueSize)
 	return false
 }
 
 // Delete a given key from the map. No-op when the key does not exist in the
 // map.
+//
 //go:nobounds
 func hashmapDelete(m *hashmap, key unsafe.Pointer, hash uint32) {
 	if m == nil {
@@ -306,28 +360,23 @@ func hashmapDelete(m *hashmap, key unsafe.Pointer, hash uint32) {
 		// > no-op.
 		return
 	}
-	numBuckets := uintptr(1) << m.bucketBits
-	bucketNumber := (uintptr(hash) & (numBuckets - 1))
-	bucketSize := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*8
-	bucketAddr := uintptr(m.buckets) + bucketSize*bucketNumber
-	bucket := (*hashmapBucket)(unsafe.Pointer(bucketAddr))
 
-	tophash := uint8(hash >> 24)
-	if tophash < 1 {
-		// 0 means empty slot, so make it bigger.
-		tophash += 1
-	}
+	tophash := hashmapTopHash(hash)
+	bucket := hashmapBucketAddrForHash(m, hash)
 
 	// Try to find the key.
 	for bucket != nil {
-		for i := uintptr(0); i < 8; i++ {
-			slotKeyOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*uintptr(i)
-			slotKey := unsafe.Pointer(uintptr(unsafe.Pointer(bucket)) + slotKeyOffset)
+		for i := uint8(0); i < 8; i++ {
+			slotKey := hashmapSlotKey(m, bucket, i)
 			if bucket.tophash[i] == tophash {
 				// This could be the key we're looking for.
-				if m.keyEqual(key, slotKey, uintptr(m.keySize)) {
+				if m.keyEqual(key, slotKey, m.keySize) {
 					// Found the key, delete it.
 					bucket.tophash[i] = 0
+					// Zero out the key and value so garbage collector doesn't pin the allocations.
+					memzero(slotKey, m.keySize)
+					slotValue := hashmapSlotValue(m, bucket, i)
+					memzero(slotValue, m.valueSize)
 					m.count--
 					return
 				}
@@ -338,6 +387,7 @@ func hashmapDelete(m *hashmap, key unsafe.Pointer, hash uint32) {
 }
 
 // Iterate over a hashmap.
+//
 //go:nobounds
 func hashmapNext(m *hashmap, it *hashmapIterator, key, value unsafe.Pointer) bool {
 	if m == nil {
@@ -349,49 +399,60 @@ func hashmapNext(m *hashmap, it *hashmapIterator, key, value unsafe.Pointer) boo
 		// initialize iterator
 		it.buckets = m.buckets
 		it.numBuckets = uintptr(1) << m.bucketBits
+		it.startBucket = uintptr(fastrand()) & (it.numBuckets - 1)
+		it.startIndex = uint8(fastrand() & 7)
+
+		it.bucketNumber = it.startBucket
+		it.bucket = hashmapBucketAddr(m, it.buckets, it.bucketNumber)
+		it.bucketIndex = it.startIndex
 	}
 
 	for {
+		// If we've wrapped and we're back at our starting location, terminate the iteration.
+		if it.wrapped && it.bucketNumber == it.startBucket && it.bucketIndex == it.startIndex {
+			return false
+		}
+
 		if it.bucketIndex >= 8 {
 			// end of bucket, move to the next in the chain
 			it.bucketIndex = 0
 			it.bucket = it.bucket.next
 		}
+
 		if it.bucket == nil {
-			if it.bucketNumber >= it.numBuckets {
-				// went through all buckets
-				return false
-			}
-			bucketSize := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*8
-			bucketAddr := uintptr(it.buckets) + bucketSize*it.bucketNumber
-			it.bucket = (*hashmapBucket)(unsafe.Pointer(bucketAddr))
 			it.bucketNumber++ // next bucket
+			if it.bucketNumber >= it.numBuckets {
+				// went through all buckets -- wrap around
+				it.bucketNumber = 0
+				it.wrapped = true
+			}
+			it.bucket = hashmapBucketAddr(m, it.buckets, it.bucketNumber)
+			continue
 		}
+
 		if it.bucket.tophash[it.bucketIndex] == 0 {
 			// slot is empty - move on
 			it.bucketIndex++
 			continue
 		}
 
-		bucketAddr := uintptr(unsafe.Pointer(it.bucket))
-		slotKeyOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*uintptr(it.bucketIndex)
-		slotKey := unsafe.Pointer(bucketAddr + slotKeyOffset)
-		memcpy(key, slotKey, uintptr(m.keySize))
+		// Found a key.
+		slotKey := hashmapSlotKey(m, it.bucket, it.bucketIndex)
+		memcpy(key, slotKey, m.keySize)
 
 		if it.buckets == m.buckets {
 			// Our view of the buckets is the same as the parent map.
 			// Just copy the value we have
-			slotValueOffset := unsafe.Sizeof(hashmapBucket{}) + uintptr(m.keySize)*8 + uintptr(m.valueSize)*uintptr(it.bucketIndex)
-			slotValue := unsafe.Pointer(bucketAddr + slotValueOffset)
-			memcpy(value, slotValue, uintptr(m.valueSize))
+			slotValue := hashmapSlotValue(m, it.bucket, it.bucketIndex)
+			memcpy(value, slotValue, m.valueSize)
 			it.bucketIndex++
 		} else {
 			it.bucketIndex++
 
 			// Our view of the buckets doesn't match the parent map.
 			// Look up the key in the new buckets and return that value if it exists
-			hash := m.keyHash(key, uintptr(m.keySize), m.seed)
-			ok := hashmapGet(m, key, value, uintptr(m.valueSize), hash)
+			hash := m.keyHash(key, m.keySize, m.seed)
+			ok := hashmapGet(m, key, value, m.valueSize, hash)
 			if !ok {
 				// doesn't exist in parent map; try next key
 				continue
@@ -409,7 +470,7 @@ func hashmapBinarySet(m *hashmap, key, value unsafe.Pointer) {
 	if m == nil {
 		nilMapPanic()
 	}
-	hash := hash32(key, uintptr(m.keySize), m.seed)
+	hash := hash32(key, m.keySize, m.seed)
 	hashmapSet(m, key, value, hash)
 }
 
@@ -418,7 +479,7 @@ func hashmapBinaryGet(m *hashmap, key, value unsafe.Pointer, valueSize uintptr)
 		memzero(value, uintptr(valueSize))
 		return false
 	}
-	hash := hash32(key, uintptr(m.keySize), m.seed)
+	hash := hash32(key, m.keySize, m.seed)
 	return hashmapGet(m, key, value, valueSize, hash)
 }
 
@@ -426,7 +487,7 @@ func hashmapBinaryDelete(m *hashmap, key unsafe.Pointer) {
 	if m == nil {
 		return
 	}
-	hash := hash32(key, uintptr(m.keySize), m.seed)
+	hash := hash32(key, m.keySize, m.seed)
 	hashmapDelete(m, key, hash)
 }
 
@@ -477,6 +538,7 @@ func hashmapStringDelete(m *hashmap, key string) {
 // is identical to the Interface() method call, except it doesn't check whether
 // a field is exported and thus allows circumventing the type system.
 // The hash function needs it as it also needs to hash unexported struct fields.
+//
 //go:linkname valueInterfaceUnsafe reflect.valueInterfaceUnsafe
 func valueInterfaceUnsafe(v reflect.Value) interface{}
 
@@ -500,7 +562,7 @@ func hashmapFloat64Hash(ptr unsafe.Pointer, seed uintptr) uint32 {
 
 func hashmapInterfaceHash(itf interface{}, seed uintptr) uint32 {
 	x := reflect.ValueOf(itf)
-	if x.RawType() == 0 {
+	if x.RawType() == nil {
 		return 0 // nil interface
 	}
 
@@ -526,10 +588,10 @@ func hashmapInterfaceHash(itf interface{}, seed uintptr) uint32 {
 	case reflect.Float64:
 		return hashmapFloat64Hash(ptr, seed)
 	case reflect.Complex64:
-		rptr, iptr := ptr, unsafe.Pointer(uintptr(ptr)+4)
+		rptr, iptr := ptr, unsafe.Add(ptr, 4)
 		return hashmapFloat32Hash(rptr, seed) ^ hashmapFloat32Hash(iptr, seed)
 	case reflect.Complex128:
-		rptr, iptr := ptr, unsafe.Pointer(uintptr(ptr)+8)
+		rptr, iptr := ptr, unsafe.Add(ptr, 8)
 		return hashmapFloat64Hash(rptr, seed) ^ hashmapFloat64Hash(iptr, seed)
 	case reflect.String:
 		return hashmapStringHash(x.String(), seed)
@@ -557,7 +619,7 @@ func hashmapInterfaceHash(itf interface{}, seed uintptr) uint32 {
 }
 
 func hashmapInterfacePtrHash(iptr unsafe.Pointer, size uintptr, seed uintptr) uint32 {
-	_i := *(*_interface)(iptr)
+	_i := *(*interface{})(iptr)
 	return hashmapInterfaceHash(_i, seed)
 }
 
